/* -*- Mode: C++ -*-
 * $Id: CoachDisplay.cc,v 1.1 2001/11/15 16:50:12 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoachDisplay.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2001-11-15
 * Last modified by $Author: frehe $ $Date: 2001/11/15 16:50:12 $
 *
 *
 * HISTORY
 *
 * $Log: CoachDisplay.cc,v $
 * Revision 1.1  2001/11/15 16:50:12  frehe
 * Added a coach agent
 *
 */

/**
 * @file CoachDisplay.cc
 * Implementation file for the class CoachDisplay.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/11/15 16:50:12 $
 */


#include <iostream>

#include "CoachDisplay.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


CoachDisplay::CoachDisplay(UInt x, UInt y)
  : viewManager(ViewManager::instance()),
    parameterView(NULL),
    coachView(NULL),
    col(x),
    row(y)
{
    getView(parameterView, PARAMETER_VIEW_ID);
    getView(coachView, COACH_VIEW_ID);
}


CoachDisplay::~CoachDisplay()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(COACH_VIEW_ID);
}


void CoachDisplay::setLastSkill(const std::string& last)
{
  lastSkill = last;
}


void CoachDisplay::draw()
{
    CLEAR_SCREEN;
    HOME_CURSOR;

    // print last skill name
    std::cout << "last skill: " << lastSkill << "\n";

    VectorUnknown brealvec = coachView->getRelativeBallVector(NOW);
    if (brealvec.getAngle().isUnknown()) {
      std::cout << "ball real direction is not known" << "\n";
    } else {
      std::cout << "ball real direction is " << brealvec.getAngle().getDeg() << "\n";
      RS_LOG( LA_AGENT, "ball real direction is " << brealvec.getAngle().getDeg() );
    }

    if (brealvec.getLength().isUnknown()) {
      std::cout << "ball real distance is not known" << "\n";
    } else {
      std::cout << "ball real distance is " << brealvec.getLength().getValue() << "\n";
    }

    PointUnknown brealpos = coachView->getBallAbsolutePosition(NOW);
    std::cout << "ball real position is " << brealpos << "\n";
    RS_LOG( LA_AGENT, "ball real position is " << brealpos );
    VectorUnknown brealspeed = coachView->getBallAbsoluteSpeed(NOW);
    std::cout << "ball real speed is " << brealspeed << "\n";
    RS_LOG( LA_AGENT, "ball real speed is " << brealspeed );

    /*
    std::vector<PlayerObject> teammates = playerView->getTeammates(NOW);
    std::cout << "Saw " << teammates.size() << " teammates." << "\n";
#ifdef RS_USE_COACH_DEBUGGING
    std::cout << "Coach saw " << coachView->getTeammates(NOW).size() << " teammates." << "\n";
#endif
    for ( std::vector<PlayerObject>::const_iterator plr = teammates.begin();
	  plr != teammates.end(); ++plr ) {
      if ( plr->getUniformNumber().isKnown() ) {
	std::cout << "Teammate no " << plr->getUniformNumber().getValue() << "\n";
	std::cout << "pos = " << PointUnknown(plr->getPosition()) << "\n";
#ifdef RS_USE_COACH_DEBUGGING
	if ( coachView->getTeammates(NOW).size() >= teammates.size() )
	  std::cout << "realpos = " << PointUnknown(coachView->getTeammates(NOW)[plr->getUniformNumber().getValue()].getPosition()) << "\n";
#endif
	std::cout << "vector = " << VectorUnknown(plr->getVector()) << "\n";
#ifdef RS_USE_COACH_DEBUGGING
	if ( coachView->getTeammates(NOW).size() >= teammates.size() )
	  std::cout << "realvector = " << VectorUnknown(coachView->getTeammates(NOW)[plr->getUniformNumber().getValue()].getVector()) << "\n";
#endif
      }
    }
    */
    
    std::cout << std::flush;
}


RS_END_NAMESPACE
