/* -*- Mode: C++ -*-
 * $Id: DribblePP.cc,v 1.5 2001/10/01 19:14:44 aip6 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DribblePP.cc
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Peder Eliasson & Peter Fornwall 2001-09-18
 * Last modified by $Author: aip6 $ $Date: 2001/10/01 19:14:44 $
 *
 *
 * HISTORY
 *
 * $Log: DribblePP.cc,v $
 * Revision 1.5  2001/10/01 19:14:44  aip6
 * Check in of DribblePP.h
 *
 * Revision 1.4  2001/09/30 17:11:02  aip6
 * Fixed a lot of bugs and introduced even more bugs.
 * Still just experimenting a lot.
 *
 * Revision 1.3  2001/09/29 18:55:19  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.2  2001/09/26 11:41:24  frehe
 * Update after move of TRUE, FALSE and UNKNOWN to inside FuzzyBool.
 *
 * Revision 1.1  2001/09/23 18:12:36  aip6
 * First version of this skill.
 * Still on experimental stage.
 *
 *
 */

/**
 * @file DribblePP.cc
 * Implementation file for the skill class DribblePP.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: aip6 $
 * $Date: 2001/10/01 19:14:44 $
 */

#include "DribblePP.h"
#include <iostream.h>
#include "BallView.h"
#include "PlayerView.h"
#include "AgentView.h"
#include "MarkerView.h"
#include "GameView.h"
#include "view_constants.h"
#include "DashCommand.h"
#include "TurnCommand.h"
#include "TurnNeckCommand.h"
#include "KickCommand.h"

RS_BEGIN_NAMESPACE


// --- CONSTRUCTORS ---

DribblePP::DribblePP()
  // This is an initializer list
  // which means that all the objects inside the
  // class can be initialized after the colon.
  // You initialize the variable myVar with
  // myVar(constructor arguments).
  // You should also initialize the base classes, but
  // only the classes that you directly inherit from.
  // If you do not initialize a variable its default
  // constructor will be called.
  : Skill(),
    ballView(NULL),
    parameterView(NULL),
    playerView(NULL),
    agentView(NULL),
    markerView(NULL),
    gameView(NULL)    
{
  std::cerr << "A DribblePP initialized without arguments" << std::endl;
  getView(ballView, BALL_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(playerView, PLAYER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
  getView(markerView, MARKER_VIEW_ID, parameterView);
  getView(gameView, GAME_VIEW_ID);
}

// --- DESTRUCTOR ---

DribblePP::~DribblePP()
{
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
}


// --- METHODS ---

Plan DribblePP::generatePlan()
{
  static Command* cmd;
  static Plan plan;
  Step step;
  static int phase = 0;
  static AngleDegUnknown old_ball_angle;
  static float dribble_dir = -2;

  //  static PointUnknown agentPosition;
  //static AngleDegUnknown toDirection, bodyDirection;
  //static AngleDeg angle;
  //static FloatUnknown distanceToPosition;

  //  if(applicable().isFalse())

  AngleDegUnknown ball_angle = ballView->getRelativeBallDirection(NOW);
  FloatUnknown ball_distance = ballView->getBallDistance(NOW);
  PointUnknown wanted_pos;

  static Point last_kick_point;
  static int ticks_left;

  // Phase 0, find the ball and turn towards it
  if (phase == 0)
  {
      if (ball_angle.isUnknown()) {
	  cmd = new TurnCommand(45);
	  step.addCommand(cmd);
	  delete cmd;
      }
      else
      {
	  PointUnknown ball_pos = ballView->getBallPosition(NOW);
	  wanted_pos = ball_pos;
	  wanted_pos.addX(-10);
	  PointUnknown agentPos = agentView->getAgentPosition(NOW);
	  AngleDegUnknown pos_dir = agentPos.getDirectionTo(wanted_pos);
	  float pos_dist = agentPos.getDistanceTo(wanted_pos).getValue();

          cmd = new TurnCommand(pos_dir.getAngle());
          step.addCommand(cmd);
	  delete cmd;
	  phase = 1;

	  /*
	  if ((abs(ball_angle) > 20) && (ball_distance > 10)) {
	      cmd = new TurnCommand(ball_angle.getAngle());
	      step.addCommand(cmd);
	      delete cmd;
	  }
	  else if ((abs(ball_angle) > 10) && (ball_distance > 1)) {
	      cmd = new TurnCommand(ball_angle.getAngle());
	      step.addCommand(cmd);
	      delete cmd;
	  }
	  */
	  /*
	    else if (ball_distance > 3) {
	    cmd = new DashCommand(60);
	    step.addCommand(cmd);
	    delete cmd;
	    }
	    else if (ball_distance > 2) {
	    cmd = new DashCommand(20);
	    step.addCommand(cmd);
	    delete cmd;
	    }
	  */
      }
  }

  // Phase 1, run to the ball
  else if(phase == 1)
  {
      PointUnknown ball_pos = ballView->getBallPosition(NOW);
      wanted_pos = ball_pos;
      wanted_pos.addX(-10);
      PointUnknown agentPos = agentView->getAgentPosition(NOW);
      AngleDegUnknown pos_dir = agentPos.getDirectionTo(wanted_pos);
      float pos_dist = agentPos.getDistanceTo(wanted_pos).getValue();

      if (pos_dist > 2)
      {
	  cmd = new DashCommand(60);
	  step.addCommand(cmd);
	  delete cmd;
      }
      else if (pos_dist > (1 + agentView->getSpeed(NOW).getValue()))
      {
	  cmd = new DashCommand(20);
	  step.addCommand(cmd);
	  delete cmd;
      }
      else
      {
	  AngleDegUnknown temp_angle = agentView->getBodyDirection(NOW);
	  float moment = (1 + agentView->getSpeed(NOW).getValue() * parameterView->SP_inertia_moment) * (-temp_angle.getAngle().getDeg());

	  cmd = new TurnCommand(moment);
	  step.addCommand(cmd);
	  delete cmd;
	  phase = 2;
      }
  }
  // Phase 2, kick the ball
  else if(phase == 2)
  {
      PointUnknown ball_pos = ballView->getBallPosition(NOW);
      VectorUnknown ball_speed = ballView->getBallAbsoluteSpeedVector(NOW);
      AngleDegUnknown goal_dir = markerView->getTheirGoal(NOW).getAbsoluteDirectionFrom(ball_pos);
      AngleDegUnknown body_dir = agentView->getBodyDirection(NOW);
      if (ball_distance > 1)
      {
/*
	  Point cur_ball_pos(ballView->getBallPosition(NOW).getPoint());
	  Point cur_agent_pos(agentView->getAgentPosition(NOW).getPoint());
	  Vector ball_speed(ballView->getBallAbsoluteSpeedVector(NOW).getVector());
	  Vector agent_speed(agentView->getBodyDirection(NOW).getAngle(),
			     agentView->getSpeed(NOW).getValue());
	  Point cur_agent_pos_min = cur_agent_pos;
	  Point cur_agent_pos_max = cur_agent_pos;
	  Vector agent_speed_max = agent_speed;
	  int t=0;
	  int inter=0;
	  float sum_distance;
	  RS_LOG( LA_DECISION, gameView->getCurrentTime() 
		  << ": cur_pall_pos: " << cur_ball_pos << endl
		  << ": cur_agent_pos: " << cur_agent_pos << endl
		  << ": ball_speed: " << ball_speed<< endl
		  << ": agent_speed: " << agent_speed << endl
		  << ": t: " << t << endl);
	  while (t<100 && inter==0)
	  {
	      t += 1;
	      if (agent_speed_max.getLength() < 0.4)
		  agent_speed_max += Vector(AngleDeg(agent_speed_max.getAngle()), 0.6);
	      else
		  agent_speed_max.setLength(1);

	      cur_ball_pos += ball_speed; // Assume ball-speed won't change
	      cur_agent_pos_min += agent_speed;
	      cur_agent_pos_max += agent_speed_max;

	      // Check if intersection is possible
	      Vector pos_vector = cur_agent_pos_min.getVectorTo(cur_agent_pos_max);
	      Vector ball_vector = cur_agent_pos_min.getVectorTo(cur_ball_pos);
	      AngleDeg ang_diff = ball_vector.getAngle() - pos_vector.getAngle();
	      if (ang_diff < 0)
		  ang_diff *= -1;
	      float d;
	      if (ang_diff < 90)
	      {
		  d = sin(ang_diff.getDeg()) * ball_vector.getLength();
		  RS_LOG( LA_DECISION, gameView->getCurrentTime() 
			  << ": t: " << t << endl
			  << ": ang_diff: " << ang_diff << endl
			  << ": d: " << d << endl
			  << ": cur_agent_pos_min: " << cur_agent_pos_min << endl
			  << ": cur_agent_pos_max: " << cur_agent_pos_max << endl
			  << ": agent_speed: " << agent_speed << endl
			  << ": agent_speed_max: " << agent_speed_max << endl
			  << ": pos_vector: " << pos_vector << endl
			  << ": ball_vector: " << ball_vector << endl
			  << ": cur_ball_pos " << cur_ball_pos << endl);
		  if (d < 1)
		  {
		      if (ball_vector.getLength() < pos_vector.getLength())
		      {
			  inter = 1;
			  sum_distance = cur_agent_pos.getDistanceTo(cur_ball_pos);
		      }
		  }
	      }

	      // Update speed of ball and player
	      ball_speed *= parameterView->SP_ball_decay;
	      agent_speed *= parameterView->SP_player_decay;
	      agent_speed_max *= parameterView->SP_player_decay;
	  }
	  RS_LOG( LA_DECISION, gameView->getCurrentTime() 
		  << ": inter: " << inter << endl
		  << ": t: " << t << endl
		  << ": sum_distance: " << sum_distance << endl);
	  if (inter == 0)
	  {
	      AngleDeg turn_angle;
	      if (ballView->getRelativeBallDirection(NOW).isUnknown())
		  turn_angle = 20;
	      else
		  turn_angle = ballView->getRelativeBallDirection(NOW).getAngle();
	      cmd = new TurnCommand(turn_angle);
	      step.addCommand(cmd);
	      delete cmd;
	  }
	  else
	  {
	      float avg_speed = sum_distance / t;
	      float needed_acc = avg_speed - agent_speed.getLength();
	      float dash_power = needed_acc / parameterView->SP_dash_power_rate;
	      cmd = new DashCommand(dash_power);
//      cmd = new KickCommand(30, -old_ball_angle.getAngle()/15);
	      step.addCommand(cmd);
	      delete cmd;
	  }
*/
	  float dash_power = 20;
	  cmd = new DashCommand(dash_power);
//      cmd = new KickCommand(30, -old_ball_angle.getAngle()/15);
	  step.addCommand(cmd);
	  delete cmd;
      }
      else
      {
	  PointUnknown agent_pos_u = agentView->getAgentPosition(NOW);
	  Point agent_pos;
	  if (agent_pos_u.isUnknown())
	      agent_pos = Point(0,0);
	  else
	      agent_pos = agent_pos_u.getPoint();
	  Point wanted_point = agent_pos + Point(5, dribble_dir);
	  dribble_dir *= -1;

	  float kick_power;
	  float kick_angle;
	  kickToTimePosition(wanted_point, 15, kick_power, kick_angle);

	  cmd = new KickCommand(kick_power, kick_angle);
	  step.addCommand(cmd);
	  delete cmd;
	  last_kick_point = wanted_point;
	  phase = 21;
      }
  }
  else if (phase == 21)
  {
      PointUnknown agent_pos_u = agentView->getAgentPosition(NOW);
      Point agent_pos;
      if (agent_pos_u.isUnknown())
	  agent_pos = Point(0,0);
      else
	  agent_pos = agent_pos_u.getPoint();

      AngleDeg wanted_dir = agent_pos.getDirectionTo(last_kick_point);

      AngleDegUnknown agent_body_u = agentView->getBodyDirection(NOW);
      AngleDeg agent_body;
      if (agent_body_u.isUnknown())
	  agent_body = AngleDeg(0);
      else
	  agent_body = agent_body_u.getAngle();


      float moment = (1 + agentView->getSpeed(NOW).getValue() * parameterView->SP_inertia_moment) * (wanted_dir - agent_body).getDeg();

      cmd = new TurnCommand(moment);
      step.addCommand(cmd);
      delete cmd;
      ticks_left = 13;
      phase = 22;
  }
  else if (phase == 22)
  {
      PointUnknown agent_pos_u = agentView->getAgentPosition(NOW);
      Point agent_pos;
      if (agent_pos_u.isUnknown())
	  agent_pos = Point(0,0);
      else
	  agent_pos = agent_pos_u.getPoint();

      float dist_to_point = agent_pos.getDistanceTo(last_kick_point);
      float avg_speed = dist_to_point / ticks_left;

      FloatUnknown agent_speed_u = agentView->getSpeed(NOW);
      float agent_speed;
      if (agent_speed_u.isUnknown())
	  agent_speed = 0;
      else
	  agent_speed = agent_speed_u.getValue();
      float wanted_acc = avg_speed - agent_speed;

      float dash_power = wanted_acc / parameterView->SP_dash_power_rate;
      RS_LOG( LA_DECISION, gameView->getCurrentTime() 
	      << ": wanted_acc: " << wanted_acc << endl
	      << ": dpr: " << parameterView->SP_dash_power_rate << endl
	      << ": dp: " << dash_power << endl);
      cmd = new DashCommand(dash_power);
      step.addCommand(cmd);
      delete cmd;
      ticks_left -= 1;
      if (ticks_left == 0 || ballView->getBallDistance(NOW) < 1)
	  phase = 2;
  }
  // Phase 3, find the ball again and turn towards it
  else if(phase == 3)
  {
      PointUnknown ball_pos = ballView->getBallPosition(NOW);
      VectorUnknown ball_speed = ballView->getBallAbsoluteSpeedVector(NOW);
      AngleDegUnknown goal_dir = markerView->getTheirGoal(NOW).getAbsoluteDirectionFrom(ball_pos);
      AngleDegUnknown body_dir = agentView->getBodyDirection(NOW);
//        if (ball_distance > 1)
//        {
//  	  cmd = new DashCommand(60);
//  	  step.addCommand(cmd);
//  	  delete cmd;
//        }
      if (1)
      {
	  float player_size = parameterView->SP_player_size;
	  float ball_size = parameterView->SP_ball_size;
	  float kickable_margin = parameterView->SP_kickable_margin;
	  float dir_diff = abs(ball_angle.getAngle().getDeg());
	  RS_LOG( LA_DECISION, gameView->getCurrentTime() 
		  << "p_size: " << player_size << endl
		  << "b_size: " << ball_size << endl
		  << "b_distance: " << ball_distance.getValue() << endl
		  << "km: " << kickable_margin << endl
		  << "dir_diff: " << dir_diff << endl);
	  float kick_power_rate = parameterView->SP_kick_power_rate *
	      (1 - (0.25 * dir_diff)/180
	       - (0.25 * (ball_distance.getValue() - player_size - ball_size)/kickable_margin));

	  Point cur_ball_speed(ball_speed.getVector());
	  Point wanted_ball_speed(0.5, 0);
	  Point ball_acc = wanted_ball_speed - cur_ball_speed;
	  float kick_power = Vector(ball_acc).getLength() / kick_power_rate;
	  float kick_angle = Vector(ball_acc).getAngle().getDeg();

	  if (ball_distance > 1)
	  {
	      cmd = new DashCommand(20);
//      cmd = new KickCommand(30, -old_ball_angle.getAngle()/15);
	      step.addCommand(cmd);
	      delete cmd;
	  }
	  else
	  {
	      cmd = new KickCommand(kick_power, kick_angle);
//      cmd = new KickCommand(30, -old_ball_angle.getAngle()/15);
	      step.addCommand(cmd);
	      delete cmd;
	      phase = 3;
	  }
//	  cmd = new KickCommand(100, -90);
//	  cmd = new KickCommand(20, 0);
//	  step.addCommand(cmd);
//	  delete cmd;
      }
/*
    AngleDegUnknown abs_ball_dir = ballView->getAbsoluteBallDirection(NOW);
    if (abs_ball_dir.isUnknown()) {
      cmd = new TurnCommand((old_ball_angle + 45).getAngle());
      step.addCommand(cmd);
      delete cmd;
    }
    else {
      cmd = new TurnCommand((ballView->getAbsoluteBallDirection(NOW) - agentView->getBodyDirection(NOW)).getAngle());
      step.addCommand(cmd);
      delete cmd;
    }
    cmd = new TurnNeckCommand(-agentView->getNeckDirection(NOW).getAngle());
    step.addCommand(cmd);
    delete cmd;
    phase = 0;
    unknown_ticks = 0;
    known_ticks = 0;
*/
  }
  old_ball_angle = ball_angle;

/*
    if(goal_dir.isKnown()) {
      cmd = new KickCommand(20, goal_dir.getAngle() - body_dir.getAngle() + dribbledir);
      step.addCommand(cmd);
      delete cmd;
      //      cmd = new TurnNeckCommand(ballView->getRelativeBallDirection(NOW).getAngle());
      //      step.addCommand(cmd);
      //      delete cmd;
      dribbledir = -1 * dribbledir;
    }
*/

/*
  else if(applicable().isTrue()){
    //setHeadDirectionToPosition();
    agentPosition = agentView->getAgentPosition(NOW);
    toDirection = agentPosition.getDirectionTo(getPosition());
    if((bodyDirection = agentView->getBodyDirection(NOW)).isKnown() &&
       ((abs(angle = toDirection.getAngle() - bodyDirection.getAngle()) > 5)))
      cmd = new TurnCommand(angle);
    else
      cmd = new DashCommand(getDash()); 
  }
  // If no command has been choosen, that is, applicable is FuzzyBool(0.5).
.-  if(cmd == NULL) cmd = new DashCommand(25); //better to do than not
*/

  cmd = NULL;
  plan = Plan(step);

  return plan;

  // Return an empty plan.
  //return Plan();
}


void DribblePP::kickToTimePosition(Point to_point,
				  int at_time,
				  float &kick_power,
				  float &kick_angle)
{
// Calculate kick_power_rate
    float player_size = parameterView->SP_player_size;
    float ball_size = parameterView->SP_ball_size;
    float kickable_margin = parameterView->SP_kickable_margin;

    AngleDegUnknown ball_angle = ballView->getRelativeBallDirection(NOW);
    float dir_diff;
    if (ball_angle.isUnknown())
	dir_diff = 0;
    else
	dir_diff = abs(ball_angle.getAngle().getDeg());

    FloatUnknown ball_distance = ballView->getBallDistance(NOW);
    float ball_dist;
    if (ball_distance.isUnknown())
	ball_dist = 0;
    else
	ball_dist = ball_distance.getValue();

    float kick_power_rate = parameterView->SP_kick_power_rate *
	(1 - (0.25 * dir_diff)/180
	 - (0.25 * (ball_dist - player_size - ball_size)/kickable_margin));

// Get current ball-position
    PointUnknown cur_ball_pos_u = ballView->getBallPosition(NOW);
    Point cur_ball_pos;
    if (cur_ball_pos_u.isUnknown())
	cur_ball_pos = Point(0,0);
    else
	cur_ball_pos = cur_ball_pos_u.getPoint();

// Get current ball-speed
    VectorUnknown cur_ball_speed_u = ballView->getBallAbsoluteSpeedVector(NOW);
    Vector cur_ball_speed;
    if (cur_ball_speed_u.isUnknown())
	cur_ball_speed = Vector(0,0);
    else
	cur_ball_speed = cur_ball_speed_u.getVector();

// Find vector from ball to wanted destination-point
    Vector ball_to_point = cur_ball_pos.getVectorTo(to_point);
    float ball_to_point_distance = ball_to_point.getLength();

// Calculate wanted speed
    float wanted_speed = ball_to_point_distance * (1 - parameterView->SP_ball_decay) /
	(1 - pow(parameterView->SP_ball_decay, at_time));
    Vector wanted_speed_vec = ball_to_point;
    wanted_speed_vec.setLength(wanted_speed);
    Vector wanted_acc = wanted_speed_vec - cur_ball_speed;

    RS_LOG( LA_DECISION, gameView->getCurrentTime() 
	    << ": cur_ball_speed: " << cur_ball_speed << endl
	    << ": cur_ball_pos: " << cur_ball_pos << endl
	    << ": ball_to_point_distance: " << ball_to_point_distance << endl
	    << ": cur_ball_pos: " << cur_ball_pos << endl
	    << ": wanted_speed: " << wanted_speed << endl
	    << ": wanted_speed_vec: " << wanted_speed_vec << endl
	    << ": wanted_acc: " << wanted_acc << endl
	    << ": to_point: " << to_point << endl);

    AngleDegUnknown agent_body_u = agentView->getBodyDirection(NOW);
    AngleDeg agent_body;
    if (agent_body_u.isUnknown())
	agent_body = AngleDeg(0);
    else
	agent_body = agent_body_u.getAngle();


    kick_power = wanted_acc.getLength() / kick_power_rate;
    kick_angle = (wanted_acc.getAngle() - agent_body).getDeg();
}

FuzzyBool DribblePP::persistent()
{
  // Never persistent.
  return FuzzyBool(0.0);
}


FuzzyBool DribblePP::applicable()
{
  // Always applicable.
  return FuzzyBool(1.0);
}

FuzzyBool DribblePP::succeed()
{
  // Always succedes.
  return FuzzyBool(1.0);
}

FuzzyBool DribblePP::failed()
{
  // Never fails.
  return FuzzyBool(0.0);
}



RS_END_NAMESPACE


// Remove from here
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "DribblePP.icc"
#endif
// To here if you do not have any inline definitions!

