/* -*- Mode: C++ -*-
 * $Id: DribblePP.h,v 1.2 2001/10/01 19:14:44 aip6 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: DribblePP.h
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Peder Eliasson & Peter Fornwall 2001-09-18
 * Last modified by $Author: aip6 $ $Date: 2001/10/01 19:14:44 $
 *
 *
 * HISTORY
 *
 * $Log: DribblePP.h,v $
 * Revision 1.2  2001/10/01 19:14:44  aip6
 * Check in of DribblePP.h
 *
 * Revision 1.1  2001/09/23 18:12:37  aip6
 * First version of this skill.
 * Still on experimental stage.
 *
 */

/**
 * @file DribblePP.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip6 $
 * $Date: 2001/10/01 19:14:44 $
 */


#ifndef DRIBBLEPP_H
#define DRIBBLEPP_H


#include "Skill.h"


RS_BEGIN_NAMESPACE

class BallView;
class AgentView;
class PlayerView;
class ParameterView;
class MarkerView;
class GameView;


/**
 * @class DribblePP DribblePP.h
 * This is the best dribble skill!
 * This is the worst dribble skill.
 * But it is better than DribbleMM.
 *
 * @pre The preconditions for the skill are ...
 * @post The intended postconditions for the skill are...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: aip6 $
 * $Date: 2001/10/01 19:14:44 $
 */
class DribblePP : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  DribblePP();

  /** Destructor should clean up after the skill. */
  virtual ~DribblePP();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

    void kickToTimePosition(Point to_point, int at_time, float &kick_power, float &kick_angle);

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed();

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed();

  
private:
  /** Hide copy-constructor. */
  DribblePP(const DribblePP&);
  /** Hide assignment operator. */
  DribblePP& operator= (const DribblePP&);

protected:
  BallView* ballView;
  ParameterView* parameterView;
  PlayerView* playerView;
  AgentView* agentView;
  MarkerView* markerView;  
  GameView* gameView;
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "DribblePP.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !DRIBBLEPP_H
