/* -*- Mode: C++ -*-
 * $Id: FFGotoBall.cc,v 1.2 2001/09/29 18:55:20 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FFGotoBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: frehe $ $Date: 2001/09/29 18:55:20 $
 *
 *
 * HISTORY
 * $Log: FFGotoBall.cc,v $
 * Revision 1.2  2001/09/29 18:55:20  frehe
 * Removed TRUE, FALSE, and UNKNOWN.
 *
 * Revision 1.1  2001/09/28 14:27:39  andbo928
 * Imported FFGotoBall from FuzzyFoo source.
 *
 * Revision 1.12  2001/07/30 16:49:35  andbo928
 * Less jumpy behaviour.
 *
 * Revision 1.11  2001/07/30 12:17:48  mikbr212
 * Now simply runs towards ball
 *
 * Revision 1.10  2001/07/29 17:06:53  mikbr212
 * Better against cmu
 *
 * Revision 1.9  2001/07/29 14:18:29  andbo928
 * Doesn't chase as far into the future.
 *
 * Revision 1.8  2001/07/28 14:51:19  andbo928
 * Now defaults to lookAtBall=true
 *
 * Revision 1.7  2001/07/16 15:40:19  andbo928
 * Removed some stuff in applicable.
 *
 * Revision 1.6  2001/07/06 08:59:55  mikbr212
 * Made the some things work (but not yet in the new design)
 *
 * Revision 1.5  2001/07/05 17:13:24  andbo928
 * Now actually uses the future ball pos.
 *
 * Revision 1.4  2001/07/04 14:27:18  andbo928
 * Now goes to future ball positions.
 *
 * Revision 1.3  2001/07/02 12:37:48  sigge
 * Removed old misspelled log messages that confused my M-/
 *
 * Revision 1.2  2001/04/13 18:15:44  andbo928
 * Now works without a lot of old skills.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 */

/**
 * @file FFGotoBall.cc
 * Implementation file for the skill class FFGotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: frehe $
 * $Date: 2001/09/29 18:55:20 $
 */

#include <stdio.h>
#include "FFMoveToPosition.h"
#include "FFGotoBall.h"
#include "view_constants.h"
#include "TurnCommand.h"
#include "DashCommand.h"

RS_BEGIN_NAMESPACE


FFGotoBall::FFGotoBall()
  : FFMoveToPosition()
{
  setLookAtBall(true);
} 


FFGotoBall::~FFGotoBall()
{
}


Plan FFGotoBall::generatePlan()
{
  const BallState& ball(worldStateView->getWorldState().getBall());
  const AgentState& agent(worldStateView->getWorldState().getAgent());
  
  int cycles = (int) (agent.getInterceptTime(ball) * 1.2);
  
  BallState futureBall(ball);
  futureBall.step(cycles);
  TVector futurePos = futureBall.getPosition();
//      ball.getPosition() + 
//      ball.getSpeed() * -1 * (parameterView->SP_ball_decay - 1);

  static int lastCycle = -1;
  static TVector lastPos;
  if( (lastCycle+1) == ball.getGameTime() )
  {
    futurePos = (futurePos * 0.5 + lastPos * 0.5);
  }

  lastPos = futurePos;
  lastCycle = ball.getGameTime();

//    FILE* f = fopen("gotoball.m", "at");
//    fprintf(f, "(%f+i*%f) ", 
//  	  futurePos.getX(),
//  	  futurePos.getY());
//    fclose(f);

  FFMoveToPosition::setPosition(futurePos);
  return FFMoveToPosition::generatePlan();
}

FuzzyBool FFGotoBall::persistent()
{
  return FuzzyBool(0.0);
}

FuzzyBool FFGotoBall::applicable()
{
  const BallState& ball(worldStateView->getWorldState().getBall());
  if (ball.getC() > 0.3)
    return FuzzyBool(1.0);
  return FuzzyBool(0.0);
}

FuzzyBool FFGotoBall::succeed()
{
  return FuzzyBool(1.0);
}

FuzzyBool FFGotoBall::failed()
{
  return FuzzyBool(0.0);
}

RS_END_NAMESPACE





