#ifndef FFINTERCEPT_H
#define FFINTERCEPT_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"
#include "view_constants.h"
#include "rs_log.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class FFIntercept FFIntercept.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1 $
 * $Author: andbo928 $
 * $Date: 2001/09/18 20:42:31 $
 */

class FFIntercept : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  FFIntercept(int maxStp = 20);

  /** Destructor should clean up after the skill. */
  virtual ~FFIntercept();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

  void setMaxSteps(int steps);

  void setGoalie(bool g);
  bool getGoalie();

  virtual FuzzyBool succeed();
  virtual FuzzyBool failed();

protected:

  bool possible;
  int planTime;
  Plan thePlan;
  int maxSteps;
  WorldStateView* worldView;
  ParameterView* parameterView;
  TVector offset;
  float distance;
  bool goalie;

  bool opTurn(WorldState& ws, const BallState& ball, Step& opStep);
  bool opDash(WorldState& ws, Step& opStep);

  bool goalFound(const WorldState& ws);

  /** Hide copy-constructor. */
  FFIntercept(const FFIntercept&);
  /** Hide assignment operator. */
  FFIntercept& operator= (const FFIntercept&);
};


RS_END_NAMESPACE

#endif // !FFINTERCEPT_H

