/* -*- Mode: C++ -*-
 * $Id: InterceptBall.h,v 1.7 2001/09/01 23:51:46 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InterceptBall.h
 * Unitname: Framework/skill
 * $Revision: 1.7 $
 * Created by: Rufus & Martin 2000-09-17
 * Last modified by $Author: frehe $ $Date: 2001/09/01 23:51:46 $
 *
 *
 * HISTORY
 *
 * $Log: InterceptBall.h,v $
 * Revision 1.7  2001/09/01 23:51:46  frehe
 * Updated the skill with the new abstract skill class and the new version of getValue and getAngle.
 *
 * Revision 1.6  2001/08/23 00:31:58  frehe
 * Removed dependencies.
 *
 * Revision 1.5  2000/10/09 21:36:24  johbu627
 * *** empty log message ***
 *
 * Revision 1.4  2000/09/24 14:14:32  johbu627
 * Minor changes, because of changes in SkillWithSpeed
 *
 * Revision 1.3  2000/09/19 19:16:21  johbu627
 * Some minor changes in InterceptBall.cc
 *
 * Revision 1.2  2000/09/17 22:17:28  johbu627
 * changed InterceptBall to be a child of SkillWithSpeed
 *
 */

/**
 * @file InterceptBall.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:51:46 $
 */


#ifndef INTERCEPTBALL_H
#define INTERCEPTBALL_H


#include "SkillWithSpeed.h"
#include "framework_types.h"


RS_BEGIN_NAMESPACE


class BallView;
class ParameterView;
class AgentView;


/**
 * @class InterceptBall InterceptBall.h
 * InterceptBall intercept the Ball :-)
 * It tries to get the Ball as fast as possible, depending on a given speed.
 *
 * @pre The preconditions for the skill are: the ballposition, its speed and the playerposition are known
 * @post Got the ball, the ball should be kickable
 *
 * @todo The turncommand must be fine-tuned
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.7 $
 * $Author: frehe $
 * $Date: 2001/09/01 23:51:46 $
 */

class InterceptBall : public SkillWithSpeed {
public:
  /** Default constructor. Here you can initialize the skill. */
  InterceptBall(const Float spd);

  /** Destructor should clean up after the skill. */
  virtual ~InterceptBall();

  /** Don't use this. It makes no sense!!*/
  virtual Plan generatePlan();

  /** The core-routine of the skill, with spd u give the Value of the Speed to the Skill*/
  virtual Plan generatePlan(const Float spd);

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();


  /** Return @c true (close to 1) if the skill has succeeded,
      i.e. its post conditions are satisfied. */
  virtual FuzzyBool succeed() { return FuzzyBool(0.5); };

  /** Return @c true (close to 1) if the skill has failed,
      i.e. something went wrong. */
  virtual FuzzyBool failed() { return FuzzyBool(0.5); };

  
private:

  Float speed;

  // Some views often used
  Float32 ballDecay;
  Float32 playerDecay;
  Float32 powerRate;
  Float32 recoveryMin;
  Float32 staminaIncMax;
  Float32 staminaMax;
  Float32 effortMin;
  Float32 effortDecThr;
  Float32 stamina;


  BallView* ballView;
  ParameterView* parameterView;
  AgentView* agentView;


  /** Calculates the ballposition on a given Cycle */
  PointUCO ExtrapolateBallPosition(PointUCO BallNow, VectorUCO SpeedNow, FloatUCO cycel);


  /** Help-Routine */
  Float32 CalculateSpeed(Float32 usedPower, Float32 stamina, Float32 effectivity);

  /** Help-Routine */
  Float32 CalculatePower(Float32 desiredSpeed, Float32 effectivity);

  /** Help-Routine */
  Float32 CalculateTime(Float32 desiredSpeed, Float32 dist);


  /** This procedure calculates the Interception-Point */
  PointUCO InterceptBallPosition(Float32 v);


  /** Hide copy-constructor. */
  InterceptBall(const InterceptBall&);
  /** Hide assignment operator. */
  InterceptBall& operator= (const InterceptBall&);
};


RS_END_NAMESPACE


#endif // !INTERCEPTBALL_H
