#ifndef ANGLETHINGY_H
#define ANGLETHINGY_H

#include <stdio.h>
#include "library_types.h"
#include <list>
#include <vector>
#include "ViewManager.h"
#include "WorldState.h"
#include "WorldStateView.h"
#include "ParameterView.h"
#include "view_constants.h"
#include "BallState.h"

RS_BEGIN_NAMESPACE

class AngleThingy
{
public:
  struct Span {
    double start;
    double end;
  };
  
private:
  // Opponent penalty.
  static const int actionPenalty = 3;
  // How big percentage of max_speed will oppponents and teammates run in.
  static const float opponentEffect = 1.0;
  // How many cycles should be penelty due to observation lag.
  static const int observationPenalty = 2;

  WorldState worldState;
  BallState ball;
  float ballRadius;
  std::list<AngleThingy::Span> spans;
  float v0;
  int time;

  bool compensateSlide;

  FILE* matlab;
  void dumpCircle(TVector p, double r);
  void dumpSpans();
  void dumpOpponents();
  void dumpAgent();

protected:
  WorldStateView* worldView;
  ParameterView* parameterView;
  float getActionRadius();

public:
  AngleThingy();
  ~AngleThingy();
  void addSpan(const AngleThingy::Span& span);
  void removeSpan(const AngleThingy::Span& s);
  void setKickAcceleration(float a);
  void setKickPower(float p);
  void setInitialSpeed(float v);
  void setWorldState(const WorldState& ws);
  void step();
  void reset();
  const std::list<AngleThingy::Span> getSpans();
  float getBallRadius();
  const BallState& getBall();
  AngleThingy::Span getMaxSpan();
};

RS_END_NAMESPACE

#endif // ANGLETHINGY_H
