/* -*- Mode: C++ -*-
 * $Id: CoachCommand.cc,v 1.1 2001/11/15 16:48:31 frehe Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CoachCommand.cc
 * Unitname: Library
 * $Revision: 1.1 $
 * Created by: Fredrik Heintz 2001-11-15
 * Last modified by $Author: frehe $ $Date: 2001/11/15 16:48:31 $
 *
 *
 * HISTORY
 *
 * $Log: CoachCommand.cc,v $
 * Revision 1.1  2001/11/15 16:48:31  frehe
 * Added a coach command.
 *
 */

/**
 * @file CoachCommand.cc
 * Implementation of the class CoachCommand.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1 $
 * $Author: frehe $
 * $Date: 2001/11/15 16:48:31 $
 */


#include "CoachCommand.h"
#include "string_utils.h"


RS_BEGIN_NAMESPACE


CoachCommand::CoachCommand(const std::string& cmd)
  : ServerCommand(CMD_Coach),
    command(cmd)
{
}


CoachCommand::CoachCommand(const CoachCommand& obj)
  : ServerCommand(obj),
    command(obj.command)
{
}


CoachCommand::~CoachCommand()
{
}


void
CoachCommand::copyCommand(const CoachCommand& obj)
{
  ServerCommand::copyCommand(obj);
  command = obj.command;
}


CoachCommand&
CoachCommand::operator =(const CoachCommand& obj)
{
  if ( this != &obj ) {
    copyCommand(obj);
  }

  return *this;
}


bool
CoachCommand::onlyOneEachCycle() const
{
  return false;
}


bool
CoachCommand::isImmediateCommand() const
{
  return true;
}


const std::string
CoachCommand::serverString() const
{
  return command;
}


CoachCommand*
CoachCommand::clone() const
{
  return new CoachCommand(*this);
}


RS_END_NAMESPACE
